"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.makePs1Script = exports.makeCmdScript = exports.makeShScript = exports.linkTool = void 0;
const fs = require("fs");
const path = require("path");
const mixed_1 = require("./mixed");
const parse_1 = require("./parse");
function linkTool(cwd, nodeModulesBinPath, tool) {
    const linkPath = path.join(nodeModulesBinPath, tool.name);
    const relativeLinkPath = path.relative(cwd, linkPath);
    const possiblyRelativeLinkPath = relativeLinkPath.startsWith("node_modules")
        ? relativeLinkPath
        : linkPath;
    // Just like npm, these overwrite whatever links are already in
    // `node_modules/.bin/`. Most likely it’s either old links from for example
    // the `elm` npm package, or links from previous runs of this script.
    const [linkPathPresentationString, what] = parse_1.isWindows
        ? // istanbul ignore next
            [symlinkShimWindows(tool, linkPath, possiblyRelativeLinkPath), "shims"]
        : [symlink(tool, linkPath, possiblyRelativeLinkPath), "link"];
    if (linkPathPresentationString instanceof Error) {
        return new Error(linkPathPresentationString.message);
    }
    if (linkPathPresentationString === undefined) {
        return `${mixed_1.bold(`${tool.name} ${tool.version}`)}: ${mixed_1.dim("all good")}`;
    }
    return `${mixed_1.bold(`${tool.name} ${tool.version}`)} ${what} created: ${mixed_1.dim(`${linkPathPresentationString} -> ${tool.absolutePath}`)}\n${mixed_1.indent(`To run: npx ${tool.name}`)}`;
}
exports.linkTool = linkTool;
function symlink(tool, linkPath, possiblyRelativeLinkPath) {
    try {
        if (fs.readlinkSync(linkPath) === tool.absolutePath) {
            return undefined;
        }
    }
    catch (_error) {
        // Continue below.
    }
    try {
        fs.unlinkSync(linkPath);
    }
    catch (errorAny) {
        const error = errorAny;
        if (error.code !== "ENOENT") {
            return new Error(`Failed to remove old link for ${tool.name} at ${possiblyRelativeLinkPath}:\n${error.message}`);
        }
    }
    try {
        fs.symlinkSync(tool.absolutePath, linkPath);
    }
    catch (errorAny) /* istanbul ignore next */ {
        const error = errorAny;
        return new Error(`Failed to create link for ${tool.name} at ${possiblyRelativeLinkPath}:\n${error.message}`);
    }
    return possiblyRelativeLinkPath;
}
// istanbul ignore next
function symlinkShimWindows(tool, linkPath, possiblyRelativeLinkPath) {
    const items = [
        [linkPath, makeShScript(tool.absolutePath)],
        [`${linkPath}.cmd`, makeCmdScript(tool.absolutePath)],
        [`${linkPath}.ps1`, makePs1Script(tool.absolutePath)],
    ];
    const linkPathPresentationString = `${possiblyRelativeLinkPath}{,.cmd,.ps1}`;
    try {
        if (items.every(([itemPath, content]) => fs.readFileSync(itemPath, "utf8") === content)) {
            return undefined;
        }
    }
    catch (_error) {
        // Continue below.
    }
    try {
        for (const [itemPath] of items) {
            fs.unlinkSync(itemPath);
        }
    }
    catch (errorAny) {
        const error = errorAny;
        if (error.code !== "ENOENT") {
            return new Error(`Failed to remove old shims for ${tool.name} at ${linkPathPresentationString}:\n${error.message}`);
        }
    }
    try {
        for (const [itemPath, content] of items) {
            fs.writeFileSync(itemPath, content);
        }
    }
    catch (errorAny) {
        const error = errorAny;
        return new Error(`Failed to create shims for ${tool.name} at ${linkPathPresentationString}:\n${error.message}`);
    }
    return linkPathPresentationString;
}
// Windows-style paths works fine, at least in Git bash.
function makeShScript(toolAbsolutePath) {
    return lf(`
#!/bin/sh
${toolAbsolutePath
        .split(/(')/)
        .map((segment) => segment === "" ? "" : segment === "'" ? "\\'" : `'${segment}'`)
        .join("")} "$@"
`);
}
exports.makeShScript = makeShScript;
// Note: Paths on Windows cannot contain `"`.
function makeCmdScript(toolAbsolutePath) {
    return crlf(`
@ECHO off
"${toolAbsolutePath}" %*
`);
}
exports.makeCmdScript = makeCmdScript;
// The shebang is for PowerShell on unix: https://github.com/npm/cmd-shim/pull/34
function makePs1Script(toolAbsolutePath) {
    return lf(`
#!/usr/bin/env pwsh
& '${toolAbsolutePath.replace(/'/g, "''")}' $args
`);
}
exports.makePs1Script = makePs1Script;
function lf(string) {
    return `${string.trim()}\n`;
}
function crlf(string) {
    return lf(string).replace(/\n/g, "\r\n");
}
